from base64 import b64encode, b64decode
from Crypto.Cipher import AES
from Crypto.Protocol.KDF import PBKDF2


class AESCrypt:
    # 固定的 salt
    salt = b"\xd0\x18\xa7QM\xd6\x9b\xebxu\xe4\xed\xa8\x83\xf6\xa3/\x01\x9c\x9e\x86n\xda;\x10EdD\xf7\x932\xcc"

    def __init__(self, password):
        """
        建立 AESCrypt 實例
        """
        self.password = password
        # 根據密碼與 salt 產生金鑰
        self.key = self.generate_key()

    def generate_key(self):
        """
        根据密码与 salt 生成 PBKDF2 金鑰
        """
        key = PBKDF2(self.password, AESCrypt.salt, 32)
        return key

    def encrypt(self, data):
        """
        使用 AES 加密資料
        """
        # 建立加密器
        cipher = AES.new(self.key, AES.MODE_CTR)

        # 將資料加密
        ciphertext = cipher.encrypt(data)

        # 回傳加密後的資料與 nonce
        return (b64encode(ciphertext), b64encode(cipher.nonce))

    def decrypt(self, data):
        """
        使用 AES 解密資料
        """
        # 將傳入的資料解析為密文與 nonce
        ciphertext, nonce = b64decode(data[0]), b64decode(data[1])

        # 建立解密器
        cipher = AES.new(self.key, AES.MODE_CTR, nonce=nonce)

        # 解密資料並回傳
        return cipher.decrypt(ciphertext)



from .utils import get_execution_time
from .utils import get_github_repo
from .utils import decrypt_data_from_github
from .utils import update_file
from .utils import download_file
from .utils import delete_file
from .utils import get_file_list
from .utils import get_file_path
from .utils import decrypt_github_file
from .utils import get_dataset_df_data
from .utils import get_fonts
from .utils import get_stock_future_code
from .watcher import Watcher
from .shioaji_wrapper import ShioajiWrapper
from .aes_crypt import AESCrypt
from .strategymetrics import PriceMetrics
from .redis_sync import RedisSync
from .plot import create_treemap_data
from .plot import plot_tw_stock_treemap
from .finlab_data import get_finlab_data



from os import listdir
from Jlab.utils import (
    get_github_repo,
    download_file,
    # get_file_list,
)
from retry import retry
import redis
import wget
import pickle
import os


class RedisSync:
    def __init__(self, db_channel):
        self.db_channel = db_channel

    def restore(self, dataset):
        redis_conn = redis.Redis(host="127.0.0.1", port=6379, db=self.db_channel)
        with open(f"{dataset}.bin", "rb") as f:
            redis_conn.set(dataset.encode("utf-8"), f.read())
        redis_conn.close()

    def get_files(self):
        if not os.path.exists("finlab_database.csv"):
            self.wget_file(
                f"https://github.com/twfxjjbw/stockinfo/raw/main/finlab_database.csv"
            )
        with open("finlab_database.csv", "r") as f:
            finlab_db_list = f.read()
            return [item.strip() for item in finlab_db_list.split(",")]

    def delete_all_keys(self):
        redis_conn = redis.Redis(host="127.0.0.1", port=6379, db=self.db_channel)
        redis_conn.flushdb()
        redis_conn.close()

    @retry(exceptions=Exception, tries=3, delay=2, backoff=2)
    def wget_file(self, url):
        try:
            wget.download(url)
        except Exception as e:
            print(f"[ERROR] Failed to download {url}: {e}")
            raise e

    def check_update(self, dataset):
        redis_conn = redis.Redis(host="127.0.0.1", port=6379, db=self.db_channel)
        data_keys = redis_conn.keys()
        if dataset.encode("utf-8") in data_keys:
            if os.path.exists(f"{dataset}.bin"):
                os.remove(f"{dataset}.bin")
            self.wget_file(
                f"https://github.com/twfxjjbw/stockinfo/raw/main/{dataset}.bin"
            )
            with open("price:收盤價.bin", "rb") as f:
                github_df = pickle.loads(f.read())
                local_df = pickle.loads(redis_conn.get(dataset))
                redis_conn.close()
                print("\n", local_df.index.to_list()[-1].strftime("%Y-%m-%d"))
                if github_df.index.to_list()[-1].strftime(
                    "%Y-%m-%d"
                ) == local_df.index.to_list()[-1].strftime("%Y-%m-%d"):
                    return False
        else:
            redis_conn.close()
        return True

    def sync_github_to_redis(self, force_download=False, dataset=None):
        if self.check_update("price:收盤價") or force_download or dataset:
            # self.delete_all_keys()
            if dataset:
                all_files = dataset
            else:
                all_files = self.get_files()
            print(all_files)
            for file in all_files:
                self.wget_file(
                    f"https://github.com/twfxjjbw/stockinfo/raw/main/{file}.bin"
                )
                print(f" {file}")
                self.restore(file.split(".")[0])

            for file_name in listdir("."):
                if file_name.endswith(".bin"):
                    os.remove(file_name)


if __name__ == "__main__":
    redis_sync = RedisSync(db_channel=8)
    redis_sync.sync_github_to_redis(force_download=True)



import requests
import pickle
import pandas as pd


def get_finlab_data(dataset):
    url = f"https://github.com/twfxjjbw/stockinfo/raw/main/{dataset}.bin"
    r = requests.get(url)
    return pickle.loads(r.content)


if __name__ == "__main__":
    broker_mapping = get_finlab_data("broker_mapping")
    broker_transactions = get_finlab_data("broker_transactions")
    broker_transactions["date"] = pd.to_datetime(broker_transactions.index)
    broker_transactions = broker_transactions.merge(
        broker_mapping, on="broker", how="left"
    )
    broker_transactions.set_index("date", inplace=True)
    print(broker_transactions)
    selected_data = broker_transactions.loc[
        (broker_transactions.index == "2023-08-02")
        & (broker_transactions["stock_id"] == "2330")
    ]

    # selected_data = broker_transactions.loc['2023-07-28']
    print(selected_data.to_markdown())
